from os import environ

DEFAULT_BRANCH = environ.get("DEFAULT_BRANCH", "main")

SECRET_TTL = int(environ.get("SECRET_TTL", 900))

WEBHOOK_SECRET_SDB_KEY = environ.get("WEBHOOK_SECRET_SDB_KEY", "github-webhook")
WEBHOOK_SECRET_SDB_BACKEND = environ.get("WEBHOOK_SECRET_SDB_BACKEND", "secrets")



from typing import Optional

from cachetools.func import ttl_cache
from salt import config
from salt.client import LocalClient
from salt.runner import RunnerClient

from .config import SECRET_TTL

_local = LocalClient("/etc/salt/master")
_runner = RunnerClient(config.master_config("/etc/salt/master"))


@ttl_cache(ttl=SECRET_TTL)
def secret(key: str, backend: str = "secrets") -> Optional[str]:
    """
    Get a secret from SaltStack, cached for 15 minutes.
    """
    arg = f"sdb://{backend}/{key}"
    result = _runner.cmd("sdb.get", [arg])

    # Handles the case where the backend is not configured.
    if result == arg:
        return None
    return result


def sync():
    """
    Sync the SaltStack fileserver backend and modules
    """

    _runner.cmd("fileserver.update")
    _runner.cmd("saltutil.sync_all")


def apply() -> bool:
    """
    Apply the SaltStack highstate
    """

    jid = _local.cmd_async("*", "state.apply")
    return jid != 0



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['applier']

package_data = \
{'': ['*']}

install_requires = \
['cachetools>=5.3.0,<6.0.0',
 'flask>=2.2.2,<3.0.0',
 'gunicorn>=20.1.0,<21.0.0',
 'salt>=3005.1,<3006.0']

setup_kwargs = {
    'name': 'applier',
    'version': '0.1.1',
    'description': 'Automatically apply SaltStack highstate on webhooks',
    'long_description': '# applier\n\nAutomatically apply SaltStack highstate on GitHub webhooks.\n\n\n### Configuration\n\nThe following environment variables can be configured:\n\n| Environment Variable       | Default        | Description                                                    |\n|----------------------------|----------------|----------------------------------------------------------------|\n| DEFAULT_BRANCH             | main           | The branch to perform updates on, typically the default branch |\n| SECRET_TTL                 | 900            | How many seconds to cache secrets for                          |\n| WEBHOOK_SECRET_SDB_KEY     | github-webhook | The SDB secret path to read from                               |\n| WEBHOOK_SECRET_SDB_BACKEND | secrets        | The SDB secret backend to use                                  |\n',
    'author': 'Alex Krantz',
    'author_email': 'alex@krantz.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from os import environ

from flask import Flask, jsonify, request
from werkzeug.middleware.proxy_fix import ProxyFix

from .commands import apply, sync
from .config import DEFAULT_BRANCH
from .security import webhook

app = Flask(__name__)
app.wsgi_app = ProxyFix(app.wsgi_app)


@app.post("/webhook/apply")
@webhook
def apply():
    if request.json.get("ref") != "refs/heads/{DEFAULT_BRANCH}":
        return jsonify(message="not main branch, skipping")

    sync()
    apply()

    return jsonify(message="ok")



import hmac
from functools import wraps
from hashlib import sha256

from flask import abort, request

from .commands import secret
from .config import WEBHOOK_SECRET_SDB_BACKEND, WEBHOOK_SECRET_SDB_KEY


def webhook(f):
    """
    Decorator to validate webhook requests.
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        signature = request.headers.get("X-Hub-Signature-256", "").removeprefix(
            "sha256="
        )
        if not _validate(request.data, signature):
            abort(401)

        return f(*args, **kwargs)

    return wrapper


def _validate(body, signature):
    key = secret(WEBHOOK_SECRET_SDB_KEY, WEBHOOK_SECRET_SDB_BACKEND).encode()
    computed = hmac.new(key, body, sha256).hexdigest()
    return hmac.compare_digest(signature, computed)
